<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00006001VariacaoTabelaPrecoServicoService as variacaoPrecBServService;

class Sm00006001VariacaoTabelaPrecoServicoController extends SMCrud
{
    protected $dataSourceName = 'fct_variacaoprecbaseserv_widget';
    protected $variacaoPrecBServService;
     
    public function __construct ($crudService, $environment, variacaoPrecBServService $variacaoPrecBServService) {
        parent::__construct($crudService, $environment);
        $this->variacaoPrecBServService = $variacaoPrecBServService;
    }
    
    /**
     * A função "validaTipoMedidaVariavel" verifica se o tipo de medida variável
     * ja está criado para a faixa existente.
     */
    
    public function validaTipoMedidaVariavel(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrSeqItPrecBServ = $row['NRSEQITPRECBSERV']; 
            $cdTpMedidaServ = $row['CDTPMEDIDASERV']; 
            $cdtpMedidaVariavel = $row['CDTPMEDIDAVARIAVEL']; 
            $nrVaricacaoPrecBaseServ = $row['NRVARIACAOPRECBASESERV'];
            
            $ValidavariacaoPrecBServ = $this->variacaoPrecBServService->validaTipoMedidaVariavel($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $nrVaricacaoPrecBaseServ);
            
            $dtsvariacaoPrecBServ = new DataSet('VALIDACAO', $ValidavariacaoPrecBServ);
            
            $response->addDataSet($dtsvariacaoPrecBServ);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }   
    
     /**
     * A função "insereVariacaoPreco" insere os valores de variação de preço 
     * ao inserir um novo tipo de medida na variação.
     */
    
    public function insereVariacaoPreco(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrSeqItPrecBServ   = $row['NRSEQITPRECBSERV']; 
            $cdTpMedidaServ     = $row['CDTPMEDIDASERV']; 
            $cdtpMedidaVariavel = $row['CDTPMEDIDAVARIAVEL'];
            $vrPrecoVariacao    = $row['VRPRECOVARIACAO'];
            $idMultipVariacao   = $row['IDMULTIPVARIACAO'];
            $qtFranquia         = $row['QTFRANQUIA'];
            $sincronizar        = $row['SINCRONIZAR'];
            $usaFranquia        = $row['USAFRANQUIA'];
            
            $ValidavariacaoPrecBServ = $this->variacaoPrecBServService->insereVariacaoPreco($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $vrPrecoVariacao, $idMultipVariacao, $qtFranquia, $sincronizar, $usaFranquia);
            
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }   
    
    /**
     * A função "editaVariacaoPreco" edita os valores de variação de preço
     * ao alterar um tipo de medida na variação.
     */
    
    public function editaVariacaoPreco(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $idMultipVariacao   = $row['IDMULTIPVARIACAO']; 
            $nrSeqItPrecBServ   = $row['NRSEQITPRECBSERV']; 
            $cdTpMedidaServ     = $row['CDTPMEDIDASERV']; 
            $cdtpMedidaVariavel = $row['CDTPMEDIDAVARIAVEL'];        
            $qtFranquia         = $row['QTFRANQUIA'];
            $usaFranquia        = $row['USAFRANQUIA'];

            $ValidavariacaoPrecBServ = $this->variacaoPrecBServService->editaVariacaoPreco($idMultipVariacao, $nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtFranquia, $usaFranquia);
            
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }   
    
    /**
     * A função "deletaVariacaoPreco" insere os valores de variação de preço 
     * ao excluir um tipo de medida na variação.
     */
    
    public function deletaVariacaoPreco(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrSeqItPrecBServ        = $row['NRSEQITPRECBSERV']; 
            $cdTpMedidaServ          = $row['CDTPMEDIDASERV']; 
            $cdtpMedidaVariavel      = $row['CDTPMEDIDAVARIAVEL']; 

            $ValidavariacaoPrecBServ = $this->variacaoPrecBServService->deletaVariacaoPreco($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel);
            
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }  
    
    
     /**
     *  A função "validaFranquiaFaixa" verifica se existem faixas de variação cuja 
     *  franquia esteja dentro dela ou que seja menor do que a franquia.
     */
    
    public function validaFranquiaFaixa(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrSeqItPrecBServ   = $row['NRSEQITPRECBSERV']; 
            $cdTpMedidaServ     = $row['CDTPMEDIDASERV']; 
            $cdtpMedidaVariavel = $row['CDTPMEDIDAVARIAVEL']; 
            $qtFranquia         = $row['QTFRANQUIA'];
            
            $validaFranquia = $this->variacaoPrecBServService->validaFranquiaFaixa($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtFranquia);
            
            $dtsvalidaFranquia = new DataSet('VALIDACAO', $validaFranquia);
            
            $response->addDataSet($dtsvalidaFranquia);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }   
    
}
?>