<?php
namespace Teknisa\SM\Service;

use Teknisa\SM\Factory\Sm00004000ItemContratoFactory;
use Teknisa\SM\Service\Sm00015004AgendamentoDeRecursoService;
use Teknisa\SM\Util\Procedure\FctCopiaFormPrecoItCont;
use Doctrine\ORM\EntityManager;
use Teknisa\SM\Factory\SmPercentualMaximoDescontoAcrescimoOperadorFactory;
use Teknisa\SM\Factory\Sm00004000ContratoFactory;
use Teknisa\SM\Factory\Sm00001000LancamentoPropostaFactory;
use Teknisa\SM\Service\Sm00004000ContratoService;
use Teknisa\Libs\Util\NewCode;

class Sm00004000ItemContratoService {
    
    protected $itensFactory;
    protected $agendservice;
    protected $connection;
    protected $descontoAcrescimoFactory;
    protected $contratoFactory;
    protected $propostaFactory;
    protected $contratoService;
    protected $newCode;
    protected $itensEstruturaService;
    
    public function __construct(Sm00004000ItemContratoFactory $itensFactory, Sm00015004AgendamentoDeRecursoService $agendservice, EntityManager $entityManager, SmPercentualMaximoDescontoAcrescimoOperadorFactory $descontoAcrescimoFactory, Sm00004000ContratoFactory $contratoFactory, Sm00001000LancamentoPropostaFactory $propostaFactory, Sm00004000ContratoService $contratoService, NewCode $newCode, Sm00004000ContratoLocalAtendService $itensEstruturaService) {
        $this->itensFactory             = $itensFactory;
        $this->agendservice             = $agendservice;
        $this->connection               = $entityManager->getConnection();
        $this->descontoAcrescimoFactory = $descontoAcrescimoFactory;
        $this->contratoFactory          = $contratoFactory;
        $this->propostaFactory          = $propostaFactory;
        $this->contratoService          = $contratoService;
        $this->newCode                  = $newCode;
        $this->itensEstruturaService    = $itensEstruturaService;
    }
     /**
     *Valida se o percentual máximo de desconto dado pelo o vendedor esta conforme com seu perfil de acesso ou operador
     */
    public function validaPorcentualDescontoAcrescimoSave($cdservorca, $cdoperador, $idtipo, $nrorg) {
        $params = array(
            "CDOPERADOR" => $cdoperador,
            "CDSERVORCA" => $cdservorca,
            "IDTIPO"     => $idtipo,
            "NRORG"      => $nrorg
        );

        $desconto = $this->descontoAcrescimoFactory->getPercentualDescontoAcrescimo($params);
        return (count($desconto) > 0) ? $desconto[0] : null;
    }
    
    /**
     * Deleta da tabela de agendamentos pendentes os items deletados no 
     * contrato aprovado
    **/
    public function excluiItemAgendPendente($nrseqcontrato, $nritemocontrato, $nrorg){
        $params = array(
            "NRSEQCONTRATO"   => $nrseqcontrato,
            "NRITEMOCONTRATO" => $nritemocontrato,
            "NRORG"           => $nrorg
        );
        
        $this->itensFactory->excluiItemAgendPendente($params);
    }
    
    /**
     * Atualiza o tabela de agendamentos pendentes caso o item de um contrato aprovado
     * seja editado
    **/
    public function atualizaQuantidadeItensAgendPendente($nrseqcontrato, $nritemocontrato, $nrorg, $quantidade, $cdoperador, $situacao){
        $quantidadeAgend = $this->getQuantidadeItensAgendados($nrseqcontrato, $nritemocontrato, $nrorg)["QTITENSAGEND"];

        if(($quantidade - $quantidadeAgend) <= 0){
            $this->excluiItemAgendPendente($nrseqcontrato, $nritemocontrato, $nrorg);
        } else {
            $existeItem = $this->getItemAgendamentoPendente($nrseqcontrato, $nritemocontrato, $nrorg)["QUANTIDADEITEM"];
            
            if($existeItem > 0){
                $this->alteraValorQuantidadeAgendPendente($nrseqcontrato, $nritemocontrato, $nrorg, ($quantidade - $quantidadeAgend));
            } else {
                $this->agendservice->salvarAgendamentosPendentes($nrseqcontrato, null, $nrorg, $cdoperador, $nritemocontrato, null, ($quantidade - $quantidadeAgend), $situacao); 
            }
        }
    }
    
    /**
     * Atualiza o tabela de agendamentos pendentes caso a quantidade de um item 
     * seja editado 
    **/
    public function alteraValorQuantidadeAgendPendente($nrseqcontrato, $nritemocontrato, $nrorg, $quantidade){
        $params = array(
            "NRSEQCONTRATO"   => $nrseqcontrato,
            "NRITEMOCONTRATO" => $nritemocontrato,
            "QTPENDENCIA"     => $quantidade,
            "NRORG"           => $nrorg
        );
        
        $this->itensFactory->alteraValorQuantidadeAgendPendente($params);
    }
    
    /**
     * retorna a quantidade de itens que já estao agendados
    **/
    public function getQuantidadeItensAgendados($nrseqcontrato, $nritemocontrato, $nrorg){
        $params = array(
            "NRSEQCONTRATO"   => $nrseqcontrato,
            "NRITEMOCONTRATO" => $nritemocontrato,
            "NRORG"           => $nrorg
        );
        
        return $this->itensFactory->getQuantidadeItensAgendados($params)[0];
    }
    
    public function getItemAgendamentoPendente($nrseqcontrato, $nritemocontrato, $nrorg){
        $params = array(
            "NRSEQCONTRATO"   => $nrseqcontrato,
            "NRITEMOCONTRATO" => $nritemocontrato,
            "NRORG"           => $nrorg
        );
        
        return $this->itensFactory->getItemAgendamentoPendente($params)[0];
    }
    
    /**
     * Deleta as formações de preço que foram alteradas mas não utilizadas pelo usuário
    **/
    public function deletaAlteracoesInutiFormCont($nrseqcontrato, $nritemocontrato, $nrorg){
        $params = array(
            "NRSEQCONTRATO"   => $nrseqcontrato,
            "NRITEMOCONTRATO" => $nritemocontrato,
            "NRORG"           => $nrorg
        );
        
        $this->itensFactory->deletaAlteracoesInutiFormCont($params);
    }
    
    /**
     * Executa a procedure que insere a nova formação e altera o 
     * código do pai para o novo inserido
    **/
    public function executarProcedureFctCopiaFormPrecoItCont($nrformprecoservico, $nrorg, $operador, $nrseqcontrato, $nritemocontrato){
        $procedure  = new FctCopiaFormPrecoItCont($this->connection);
        $idFormacao = $procedure->executar($nrformprecoservico, $nrorg, $operador, $nrseqcontrato, $nritemocontrato);
    
        return $idFormacao;
    }
    
    /**
     * Insere na formação de Preço auxiliar o número do item criado
     * durante a inserção do item no contrato
    **/
    public function editaItensFormacaoAuxCont($rows, $nrorg, $operador){
        if(is_array($rows)){
            foreach ($rows as $row) {
                $iditemcalculo = implode("','", $row["IDITEMCALCULO"]);
                $params = array(
                    "NRFORMPRECOSERVICO" => $row["NRFORMPRECOSERVICO"],
                    "NRSEQCONTRATO"      => $row["NRSEQCONTRATO"],
                    "NRITEMOCONTRATO"    => $row["NRITEMOCONTRATO"],
                    "NRORG"              => $nrorg
                );
                
                $this->itensFactory->editaItensFormacaoAuxCont($params, $iditemcalculo);
            }
        }
    } 
    
    /**
     * Deleta o item de formação de preço inserido quando a rotina de alteração do número 
     * do item na tabela de formção de preço auxiliar não funciona
    **/
    public function deletarItensFormacao($nrseqcontrato, $nritemocontrato, $nrorg){
        $nritemocontrato = implode(",", $nritemocontrato);
        $params = array(
            "NRSEQCONTRATO"      => $nrseqcontrato,
            "NRORG"              => $nrorg
        );
        
        $this->itensFactory->deletarItensFormacao($params, $nritemocontrato);
    }

    /**
     * deleta a árvore completa de elementos alterados na formação de 
     * preço auxiliar já inserida no contrato
     **/
    public function deletarArvoreFormPrecoItCont($nrseqcontrato, $nritemocontrato, $nrorg, $nrformprecoservico, $nrcalcformprec){
        $params = array(
            "NRSEQCONTRATO"      => $nrseqcontrato,
            "NRITEMOCONTRATO"    => $nritemocontrato,
            "NRORG"              => $nrorg,
            "NRCALCFORMPRECSERV" => $nrcalcformprec,
            "NRFORMPRECOSERVICO" => $nrformprecoservico
        );
        $this->itensFactory->deletarArvoreFormPrecoItCont($params);
    }
    /**
     * Dá um update nos valores da proposta após a edição 
     * do item de contratos de composição se serviço
    **/
    public function salvarItemComposicaoProposta($rowContrato, $rowProposta){
        $rowItemProp = array(
            "NRSEQORCAMENTO"  => $rowProposta["NRSEQORCAMENTO"],
            "NRITEMORCAMENTO" => $rowProposta["NRITEMORCAMENTO"],
            "QTMEDIDAORCA"    => $rowContrato["QTMEDIDACONTR"],
            "VRUNITITEMORCA"  => $rowContrato["VRUNITITEMCONT"],
            "VRTOTALITEMORCA" => $rowContrato["VRTOTITEMCONT"],
            "VRITEMSEMDESC"   => $rowContrato["VRTOTITEMCONT"],
            "CDSERVORCA"      => $rowContrato["CDSERVORCA"]
        );
        $this->itensFactory->salvarItemComposicaoProposta($rowItemProp);
    }
    /**
     * manda atualizar o valor de um item do contrato de composição de servico
    **/
    public function atualizarItemContComposicao($nrorg, $cdoperador, $nrseqcontrato, $nrseqorcamento) {
        $paramsProposta = array(
            "NRSEQORCAMENTO" => $nrseqcontrato,
            "NRORG"          => $nrorg
        );
        $itensProposta = $this->itensFactory->getItensProposta($paramsProposta);
        foreach($itensProposta as $item){
            $params = array(
                "NRSEQCONTRATO"   => $nrseqcontrato,
                "NRORG"           => $nrorg,
                "CDOPERADOR"      => $cdoperador,
                "VRTOTALITEMCONT" => $item["VRTOTALITEMORCA"],
                "VRUNITITEMCONT"  => $item["VRUNITITEMORCA"],
                "QTITEMCONTRATO"  => $item["QTITEMORCAMENTO"],
                "CDSERVORCA"      => $item["CDSERVORCA"]
            );
            $this->itensFactory->atualizarItemContComposicao($params);
        }
    }

    public function verificarSeExisteDescontoNegociacaoContrato($nrnegociacaocontrato, $nrorg) {
        $params = array(
            "NRNEGOCIACAOCONTRATO" => $nrnegociacaocontrato,
            "NRORG"                => $nrorg
        );
        
        $dataset = $this->itensFactory->verificarSeExisteDescontoNegociacaoContrato($params);
        return ((float)$dataset[0]["VRPERCDESCITEM"] > 0);
    }

    public function verificarSeExisteDescontoItensNegociacaoContrato($nrnegociacaocontrato, $nrorg) {
        $params = array(
            "NRNEGOCIACAOCONTRATO" => $nrnegociacaocontrato,
            "NRORG"                => $nrorg
        );

        $dataset = $this->itensFactory->getSomaPercentualDescontoItensNegociacaoContrato($params);
        return ((float)$dataset[0]["VRPERCDESCITEM"] > 0);
    }

    public function getServicoPercentualMinimoDescontoPermitidoNegociacaoContrato($nrnegociacaocontrato, $cdoperador, $nrorg) {
        $params = array(
            "NRNEGOCIACAOCONTRATO" => $nrnegociacaocontrato,
            "NRORG"                => $nrorg
        );
        $dataset = $this->itensFactory->getServicosNegociacaoContrato($params);

        $servicos = array();
        foreach ($dataset as $row) {
            $servicos[] = $row["CDSERVORCA"];
        }

        $servicos = implode(",", $servicos);
        
        $params = array(
            "CDOPERADOR" => $cdoperador,
            "NRORG"      => $nrorg
        );
        $dataset = $this->descontoAcrescimoFactory->getServicosComPercentualDescontoPermitido($params, $servicos);
        
        return (count($dataset) > 0) ? $dataset[0] : null;
    }

    public function atualizarValorItensNegociacaoValores($nrseqcontrato, $nritemocontrato, $vrtotalitemcont, $nritemcontratopai, $nrorg, $cdoperador) {
        $params = array(
            "NRITEMOCONTRATO" => $nritemocontrato,
            "NRORG"           => $nrorg
        );
        $dataset = $this->itensFactory->getCodigoNegociacaoDoItem($params);
        
        if (count($dataset) > 0) {
            $nrnegociacaocontrato = $dataset[0]["NRNEGOCIACAOCONTRATO"];
            $params = array(
                "VRTOTITEM"            => $vrtotalitemcont,
                "CDOPERULTATU"         => $cdoperador,
                "NRORGULTATU"          => $nrorg,
                "NRNEGOCIACAOCONTRATO" => $nrnegociacaocontrato,
                "NRITEMOCONTRATO"      => $nritemocontrato,
                "NRORG"                => $nrorg
            );
            $this->itensFactory->atualizarValorItemNegociacao($params);

            /* ========================================================== */
            
            //1- retorna a soma do valor dos elementos filhos do item
            $params = array(
                "NRNEGOCIACAOCONTRATO" => $nrnegociacaocontrato,
                "NRITEMCONTRATOPAI"    => $nritemcontratopai,
                "NRORG"                => $nrorg
            );
            $dataset = $this->itensFactory->getSomaValorElementosItemNegociacao($params);
            $somaValoresItem = (float)$dataset[0]["VRTOTAL"];

            //2- atualiza o item pai com a soma dos filhos
            $params = array(
                "VRTOTITEM"            => $somaValoresItem,
                "CDOPERULTATU"         => $cdoperador,
                "NRORGULTATU"          => $nrorg,
                "NRNEGOCIACAOCONTRATO" => $nrnegociacaocontrato,
                "NRITEMOCONTRATO"      => $nritemcontratopai,
                "NRORG"                => $nrorg
            );
            $this->itensFactory->atualizarValorItemNegociacao($params);
            
            /* ========================================================== */
            
            $params = array(
                "NRSEQCONTRATO" => $nrseqcontrato,
                "NRORG"         => $nrorg
            );
            $negociacoes = $this->itensFactory->getNegociacoesContrato($params);
            foreach ($negociacoes as $nego) {
                $params = array(
                    "NRNEGOCIACAOCONTRATO" => $nego["NRNEGOCIACAOCONTRATO"],
                    "NRORG"                => $nrorg
                );
                $soma = $this->itensFactory->getSomaValorItensNegociacao($params);
                $soma = $soma[0]["VRTOTAL"];
                
                $params = array(
                    "VRTOTAL"              => $soma,
                    "CDOPERULTATU"         => $cdoperador,
                    "NRORGULTATU"          => $nrorg,
                    "NRNEGOCIACAOCONTRATO" => $nego["NRNEGOCIACAOCONTRATO"],
                    "NRORG"                => $nrorg
                );
                $this->itensFactory->atualizarValorNegociacaoComSomaDosItens($params);
            }
            
            $this->contratoService->excluirParcelasNegociacao($nrnegociacaocontrato, $nrorg);
        }
    }
    /**
     * Salva alterações no nome contratual dos itens 
    **/
    public function salvarNomeContratualContrato($rows, $nrorg, $cdoperador) {
        foreach($rows as $row){
            $params = array(
                "NRSEQCONTRATO"        => $row["NRSEQCONTRATO"],
                "IDELEMENTO"           => $row["IDELEMENTO"],
                "NMCONTRATUALITEMCONT" => $row["NMCONTRATUALITEMCONT"],
                "NRORG"                => $nrorg,
                "CDOPERADOR"           => $cdoperador
            );
            $this->itensFactory->salvarNomeContratualContrato($params);
        }
    }
    /**
     * Retorna as informações de nome contratual dos itens da proposta
    **/
    public function getInfoNomeContratualOrca($contrato, $nrog, $rows) {
        $params = array(
            "NRSEQCONTRATO" => $contrato,
            "NRORG"         => $nrog
        );
        
        $nmContratualItens = array();
        $dataset = $this->itensFactory->getInfoNomeContratualOrca($params);
        foreach($rows as $row) {
            foreach($dataset as &$nmContratual) {
                if($row["IDELEMENTO"] == $nmContratual["IDELEMENTO"]) {
                    $nmContratual["NMCONTRATUALITEMORCA"] = $row["NMCONTRATUALITEMCONT"];
                    array_push($nmContratualItens, $nmContratual);
                    break;
                }
            }
        }
        return $nmContratualItens;
    }
    /**
     * Replica as alterações de um item filho para o seu pai
    **/
    public function replicarAlteracoesParaItensPais($row, $nrorg, $cdoperador) {
        $params = array (
            "CDSERVORCA"    => $row["CDSERVORCA"],
            "NRSEQCONTRATO" => $row["NRSEQCONTRATO"],
            "NRORG"         => $nrorg
        );
        $itemPaiArvore = $this->itensFactory->getItemPaiArvore($params);
        foreach($itemPaiArvore as $itemPai) {
            $vrtotalServico = $this->itensFactory->getValorTotalServico($params);
            $params = array(
                "NRSEQCONTRATO"   => $itemPai["NRSEQCONTRATO"],
                "NRITEMOCONTRATO" => $itemPai["NRITEMOCONTRATO"],
                "CDSERVORCA"      => $itemPai["CDSERVORCA"],
                "VRUNITITEMCONT"  => $vrtotalServico,
                "VRTOTITEMCONT"   => $vrtotalServico,
                "CDOPERULTATU"    => $cdoperador,
                "NRORG"           => $nrorg
            );
            $this->itensFactory->replicarAlteracoesParaItensPais($params);
        }
    }
    /**
     * retorna os itens da proposta geradas pelo contrato
    **/
    public function getItensPropostaGeradaPorContrato($row) {
        $params = array(
            "NRSEQCONTRATO"      => $row["NRSEQCONTRATO"], 
            "NRORG"              => $row["NRORG"],
            "CDSERVORCA"         => $row["CDSERVORCA"],
            "IDELEMENTO"         => $row["IDELEMENTO"],
            "CDTPMEDIDAVARIAVEL" => $row["CDTPMEDIDASERV_GRID"],
            "CDTPMEDIDASERV"     => $row["CDTPMEDIDASERV"]
        );
        return $this->itensFactory->getItensPropostaGeradaPorContrato($params);
    }
    /**
     * Valida a faixa de variação de preço do item no contrato 
    **/
    public function validarVaricaoDePrecoItens($row, $nrorg, $cdoperador) {
        $faixaPrecVarAtual = null;
        $qtMedidaCont      = $row["QTMEDIDACONTR"];
        $qtMedidaInicial   = 0;
        $qtMedidaFinal     = 0;
        
        $params = array(
            "NRORG"              => $nrorg,
            "CDTPMEDIDASERV"     => $row["CDTPMEDIDASERV"],
            "CDTPMEDIDAVARIAVEL" => $row["CDTPMEDIDASERV_GRID"]
        );
        $variacoesPreco = $this->itensFactory->getVariacoesPrecoDoServico($params, $row["CDSERVORCA"]);
        foreach ($variacoesPreco as $variacaoPreco) {
            if($variacaoPreco["QTMEDIDAINIVARIACAO"] <= $qtMedidaCont && $qtMedidaCont <= $variacaoPreco["QTMEDIDAFIMVARIACAO"]){
                $faixaPrecVarAtual = $variacaoPreco;
                break;
            } else {
                if($variacaoPreco["QTMEDIDAINIVARIACAO"] < $qtMedidaInicial || $qtMedidaInicial == 0)
                    $qtMedidaInicial = $variacaoPreco["QTMEDIDAINIVARIACAO"];
                if($variacaoPreco["QTMEDIDAFIMVARIACAO"] > $qtMedidaFinal)
                    $qtMedidaFinal = $variacaoPreco["QTMEDIDAFIMVARIACAO"];
            }
        }
       
        return $faixaPrecVarAtual;
    }

    /**
     * Insere um item de negociação de valores no contrato com um item de contrato informado. 
     * Se já existir uma negociação de valores com os mesmos parâmetros de faturamento do item, 
     * o item é inserido nela. Se não existir, é criada uma nova negociação para o item.
     */
    public function inserirItemNegociacaoValoresContrato($row, $nrorg, $cdoperador, $nrtipoorcamento = null) {
        $this->connection->beginTransaction();
        try {
            $inseriuNegociacao = false;
            
            if (empty($nrtipoorcamento))
                $nrtipoorcamento = $this->contratoService->getEscopoContrato($row["NRSEQCONTRATO"], $nrorg);

            //retorna os parâmetros de faturamento do escopo ou do item do escopo, filtrando o serviço inserido.
            $paramsFatItem = $this->getParametrosFaturamentoParaItemNegociacaoContrato($row["CDSERVORCA"], $row["NRSEQCONTRATO"], $row["NRITEMOCONTRATO"], $nrtipoorcamento, $nrorg);

            if ($paramsFatItem) {
                if ($paramsFatItem["CDFILIALFATURA"] == null) {
                    throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Unidade para Faturamento" não foi encontrado para o item "' . $row["NMITEMCONTRATO"] . '". Favor conferir a parametrização de faturamento do Escopo.');
                }
                if ($paramsFatItem["CDTIPOOPERFATURA"] == null) {
                    throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Tipo de Operação da Nota Fiscal" não foi encontrado para o item "' . $row["NMITEMCONTRATO"] . '". Favor conferir a parametrização de faturamento do Escopo.');
                }
                if ($paramsFatItem["CDSERIENF"] == null) {
                    throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Série da Nota Fiscal" não foi encontrado para o item "' . $row["NMITEMCONTRATO"] . '". Favor conferir a parametrização de faturamento do Escopo.');
                }
                if ($paramsFatItem["CDPRODFATURA"] == null) {
                    throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Produto para Faturamento" não foi encontrado para o item "' . $row["NMITEMCONTRATO"] . '". Favor conferir a parametrização de faturamento do Escopo ou do Serviço.');
                }
                
                $dadosContrato = $this->contratoService->getDadosContrato($row["NRSEQCONTRATO"], $nrorg);
                //$geraParcelasSeparadasPorLocal = $dadosContrato["IDGERARATEIO"];
                $agrupamentoParcelas = $dadosContrato["IDAGRUPPARC"];
                
                $nrestruturam = null;
                $cdpaisfat = $sgestadofat = $cdmunicipiofat = null;
                
                
                //preencher com o endereçõ da tabela EMPRESA 
                $dadosEmpresa = $this->contratoFactory->getDadosEmpresa(array(
                    "CDFILIAL" => $paramsFatItem["CDFILIALFATURA"],
                    "NRORG" => $nrorg
                ));
                $enderecofaturamento = $this->contratoService->getEnderecoFaturamento($row["NRESTRUTURAM"], $paramsFatItem["NRPARCFATURA"], $nrorg);
                
                if (count($dadosEmpresa) > 0) {
                    $dadosEmpresa = $dadosEmpresa[0];
                    $cdpaisempresa      = $dadosEmpresa["CDPAIS"];
                    $sgestadoempresa    = $dadosEmpresa["SGESTADO"];
                    $cdmunicipioempresa = $dadosEmpresa["CDMUNICIPIO"];
                }
                if ($paramsFatItem['IDMUNIPRESTSERV'] == 'U') { //Unidade
                    $cdpaisfat      = $cdpaisempresa;
                    $sgestadofat    = $sgestadoempresa;
                    $cdmunicipiofat = $cdmunicipioempresa;
                }
                else { //Tomador
                    //preencher com o endereçõ do ENDERECO PARC 
                    $cdpaisfat      = $enderecofaturamento["CDPAIS"];
                    $sgestadofat    = $enderecofaturamento["SGESTADO"];
                    $cdmunicipiofat = $enderecofaturamento["CDMUNICIPIO"];
                }
                
                if ($agrupamentoParcelas != "1") {//1 = Agrupar tudo
                    if ($agrupamentoParcelas == "2") {//2 = Agrupar por Local de Atendimento
                        $nrestruturam = $row["NRESTRUTURAM"];
                    }
                    
                    if (!empty($enderecofaturamento)) {
                        $cdpaisfat      = $enderecofaturamento["CDPAIS"];
                        $sgestadofat    = $enderecofaturamento["SGESTADO"];
                        $cdmunicipiofat = $enderecofaturamento["CDMUNICIPIO"];
                    }
                }
                
                if ($paramsFatItem["IDTIPONATUOPER"] == 'V') {
                    //comparar se o endereço é igual ao endereço da empresa 
                    if ($cdpaisfat == $cdpaisempresa && $sgestadofat == $sgestadoempresa && $cdmunicipiofat == $cdmunicipioempresa)
                        $natuOperacao = "01";
                    else
                        $natuOperacao = "02";
                }
                else {
                    $natuOperacao = $paramsFatItem["IDNATUOPER"];
                }
                
                // Busca se já existe uma negociação inserida no contrato com os mesmos parâmetros de faturamento do item.
                // Se já existe, o item será inserido nela.
                // Se não existe, insere uma nova negociação para depois inserir o item.
                $nrnegociacaocontrato = $this->contratoService->getNegociacaoContratoPorParametrosFaturamento($row["NRSEQCONTRATO"], $nrorg, $nrtipoorcamento, $paramsFatItem["IDTPFATURA"], $paramsFatItem["CDFILIALFATURA"], $paramsFatItem["NRPARCFATURA"], $paramsFatItem["CDTIPOOPERFATURA"], $paramsFatItem["CDSERIENF"], $paramsFatItem["CDEMPRESA"], $paramsFatItem["CDINSCESTA"], $paramsFatItem["CDINSCMUNI"], $agrupamentoParcelas, $paramsFatItem["IDPREVCOBRFUT"], $nrestruturam, $paramsFatItem["CDTRIBPRODPADR"], $cdpaisfat, $sgestadofat, $cdmunicipiofat, $paramsFatItem["IDNATUOPER"]);
                
                if (is_null($nrnegociacaocontrato)) {
                    $nrnegociacaocontrato = $this->newCode->getCodeByTableName("FCT_NEGOCIACAOCONTRATO", $nrorg, 1, 10, true);
                    $params = array(
                        "NRNEGOCIACAOCONTRATO" => $nrnegociacaocontrato,
                        "NRSEQCONTRATO"        => $row["NRSEQCONTRATO"],
                        "NRTIPOORCAMENTO"      => $nrtipoorcamento,
                        "IDTPFATURA"           => $paramsFatItem["IDTPFATURA"],
                        "CDFILIALFATURA"       => $paramsFatItem["CDFILIALFATURA"],
                        "NRPARCFATURA"         => $paramsFatItem["NRPARCFATURA"],
                        "CDTIPOOPERFATURA"     => $paramsFatItem["CDTIPOOPERFATURA"],
                        "CDSERIENF"            => $paramsFatItem["CDSERIENF"],
                        "CDEMPRESA"            => $paramsFatItem["CDEMPRESA"],
                        "CDINSCESTA"           => $paramsFatItem["CDINSCESTA"],
                        "CDINSCMUNI"           => $paramsFatItem["CDINSCMUNI"],
                        "VRTOTITEM"            => $row["VRTOTITEMCONT"],
                        "VRPERCDESCITEM"       => null,
                        "VRDESCITEM"           => null,
                        "VRTOTITEMCOMDESC"     => $row["VRTOTITEMCONT"],
                        "IDATIVO"              => "S",
                        "NRORG"                => $nrorg,
                        "NRORGINCLUSAO"        => $nrorg,
                        "CDOPERINCLUSAO"       => $cdoperador,
                        "NRNEGOCIACAOPROPOSTA" => null,
                        "NRESTRUTURAM"         => $nrestruturam,
                        "IDPREVCOBRFUT"        => $paramsFatItem["IDPREVCOBRFUT"],
                        "CDTRIBPRODPADR"       => $paramsFatItem["CDTRIBPRODPADR"],
                        "CDPAISFAT"            => $cdpaisfat,
                        "SGESTADOFAT"          => $sgestadofat,
                        "CDMUNICIPIOFAT"       => $cdmunicipiofat,
                        "IDNATUOPER"           => $natuOperacao 
                    );
                    $this->contratoFactory->inserirNegociacaoContrato($params);
                    $inseriuNegociacao = true;
    
                    // Se o tipo de faturamento da negociação não for parcelamento serão inseridas as competências na negociação,
                    // tendo como base a parametrização do escopo de faturamento ou do seu item.
                    if ($paramsFatItem["IDTPFATURA"] != "P") {
                        $params = array(
                            "NRTIPOORCAMENTO" => $nrtipoorcamento,
                            "CDSERVORCA"      => $row["CDSERVORCA"],
                            "NRORG"           => $nrorg
                        );
                        $competencia = $this->propostaFactory->getCompetenciaItemNegociacao($params);
                        
                        foreach ($competencia as $comp) {
                            $nrcompetencianegocont = $this->newCode->getCodeByTableName("FCT_COMPETENCIANEGOCONT", $nrorg, 1, 10, true);
                            $params = array(
                                "NRCOMPETENCIANEGOCONT" => $nrcompetencianegocont,
                                "NRNEGOCIACAOCONTRATO"  => $nrnegociacaocontrato,
                                "NRDIACOMPETINI"        => $comp["NRDIACOMPETINI"],
                                "NRDIACOMPETFIN"        => $comp["NRDIACOMPETFIN"],
                                "IDMESCOMPETINI"        => $comp["IDMESCOMPETINI"],
                                "IDMESCOMPETFIN"        => $comp["IDMESCOMPETFIN"],
                                "QTDIASAPURACAO"        => $comp["QTDIASAPURACAO"],
                                "IDTPCALCVENC"          => $comp["IDTPCALCVENC"],
                                "NRDIACALCVENC"         => $comp["NRDIACALCVENC"],
                                "IDMESCALCVENC"         => $comp["IDMESCALCVENC"],
                                "QTDIASFATUVENC"        => $comp["QTDIASFATUVENC"],
                                "NRCOMPETENCIANEGOPROP" => null,
                                "IDATIVO"               => "S",
                                "NRORG"                 => $nrorg,
                                "NRORGINCLUSAO"         => $nrorg,
                                "CDOPERINCLUSAO"        => $cdoperador
                            );
                            $this->contratoFactory->inserirCompetenciaNegociacaoContrato($params);
                        }
                    }
                }
    
                // Insere o item na negociação de valores do contrato
                $nritnegociacaocontrato = $this->newCode->getCodeByTableName("FCT_ITNEGOCIACAOCONTRATO", $nrorg, 1, 10, true);
                $params = array(
                    "NRITNEGOCIACAOCONTRATO" => $nritnegociacaocontrato,
                    "NRNEGOCIACAOCONTRATO"   => $nrnegociacaocontrato,
                    "NRITEMORCAMENTO"        => null,
                    "NRITEMORCAMENTOPAI"     => null,
                    "VRTOTITEM"              => $row["VRTOTITEMCONT"],
                    "VRPERCDESCITEM"         => null,
                    "VRDESCITEM"             => null,
                    "VRTOTITEMCOMDESC"       => $row["VRTOTITEMCONT"],
                    "CDSERVORCA"             => $row["CDSERVORCA"],
                    "CDPRODFATURA"           => $paramsFatItem["CDPRODFATURA"],
                    "NRPARCFATURA"           => $paramsFatItem["NRPARCFATURA"],
                    "IDATIVO"                => "S",
                    "NRORG"                  => $nrorg,
                    "NRORGINCLUSAO"          => $nrorg,
                    "CDOPERINCLUSAO"         => $cdoperador,
                    "NRITEMOCONTRATO"        => $row["NRITEMOCONTRATO"],
                    "NRITNEGOCIACAOPROPOSTA" => null,
                    "NRITEMCONTRATOPAI"      => isset($row["NRITEMCONTRATOPAI"]) ? $row["NRITEMCONTRATOPAI"] : null
                );
                $this->contratoFactory->inserirItemNegociacaoContrato($params);
    
                // Se não inseriu uma negociação nova, seu valor é atualizado com a soma dos itens 
                // e os valores da aba "Desconto por Período" também são atualizados.
                if (!$inseriuNegociacao) {
                    $this->contratoService->atualizarValoresNegociacaoContrato($nrnegociacaocontrato, $nrorg, $cdoperador);
                    
                    //atualiza as parcelas da negociação com o valor a mais do item que foi inserido
                    $this->contratoService->reajustarParcelasNegociacao($row["NRSEQCONTRATO"], $nrnegociacaocontrato, $nrorg, $cdoperador);
                }
            }

            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getParametrosFaturamentoParaItemNegociacaoContrato($cdservorca, $nrseqcontrato, $nritemocontrato, $nrtipoorcamento, $nrorg) {
        $params = array(
            "CDSERVORCA"      => $cdservorca,
            "NRSEQCONTRATO"   => $nrseqcontrato,
            "NRITEMOCONTRATO" => $nritemocontrato,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        $dataset = $this->itensFactory->getParametrosFaturamentoParaItemNegociacaoContrato($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }

    public function excluirItemContrato($nritemocontrato, $nrorg) {
        $params = array(
            "NRITEMOCONTRATO" => $nritemocontrato,
            "NRORG"           => $nrorg
        );

        $this->itensFactory->excluirItemContrato($params);
    }
    /**
     * Valida e altera os dados do rateio
    **/
    public function gerarRateioItensManualmente($row, $nrtipoorcamento, $nrorg) {
        $geraRateio = $this->itensEstruturaService->verificaGeraRateioContrato($nrtipoorcamento, $nrorg);
        if($geraRateio) {
            $row["VRTOTAL"] = $row["VRTOTITEMCONT"];
            $this->itensEstruturaService->geraRateios(array($row), $row["NRSEQCONTRATO"]);
        }
    }
    /**
     * Atualiza o valor dos rateios de um item 
    **/
    public function atualizarValorRateiosContrato($nrseqcontrato, $nritemocontrato, $nrnegociacaocontrato, $nrorg, $cdoperador) {
        $params = array(
            "NRITEMOCONTRATO"      => $nritemocontrato,
            "NRNEGOCIACAOCONTRATO" => $nrnegociacaocontrato,
            "NRORG"                => $nrorg
        );
        
        $itens = $this->itensFactory->getItensNegociacaoValoresContrato($params);
        foreach($itens as $item) {
            $valorTotalItem = $item["VRTOTITEMCOMDESC"];
            $params = array(
                "NRSEQCONTRATO"    => $nrseqcontrato,
                "NRITEMOCONTRATO"  => $nritemocontrato,
                "NRORG"            => $nrorg,
                "CDOPERULTATU"     => $cdoperador,
                "VRTOTALITEM"      => $valorTotalItem
            );
            $this->itensFactory->atualizarValorRateioItens($params);
        }
    }
    
    /**
     * retorna o valor do serviço na tabela de preço
    **/
    public function getValorUnitarioTblPrecoItemContrato($nrtipoorcamento, $cdservorca, $cdmedidavar, $qtdmedida, $nrorg) {
        $params = array(
            "NRTIPOORCAMENTO"    => $nrtipoorcamento,
            "CDSERVORCA"         => $cdservorca,
            "CDTPMEDIDAVARIAVEL" => $cdmedidavar,
            "QTMEDIDASERV"       => $qtdmedida,
            "NRORG"              => $nrorg
        );
        
        $vrpreco = $this->itensFactory->getValorUnitarioTblPrecoItemContrato($params);
        
        return (count($vrpreco) > 0) ? $vrpreco[0]["VRPRECO"] : null;
    }
    

    public function excluirMedidaVariavelServico($nrseqcontrato, $cdservorca, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "CDSERVORCA"    => $cdservorca,
            "NRORG"         => $nrorg
        );

        $this->itensFactory->excluirMedidaVariavelServico($params);
    }

    public function inserirOuAtualizarMedidaVariavelServico($row, $nrorg, $cdoperador, $somenteAtualizar = false) {
        $qtmedidaservico = $this->contratoService->getSomaQuantidadeMedidaServicoContrato($row["NRSEQCONTRATO"], $row["CDSERVORCA"], $nrorg);
       
        if($qtmedidaservico) $row["QTMEDIDACONTR"] = $qtmedidaservico;
        $this->contratoService->inserirOuAtualizarMedidaVariavelServico($row, $nrorg, $cdoperador, $somenteAtualizar);
    }

    public function excluirMedidasVariaveisContratoSemItemRelacionado($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->itensFactory->getMedidasVariaveisContratoSemItemRelacionado($params);
        if (count($dataset) > 0) {
            $this->excluirMedidaVariavelServico($nrseqcontrato, $dataset[0]["CDSERVORCA"], $nrorg);
        }
    }
    
    public function regerarNegociacaoValores($nrseqcontrato, $nrtipoorcamento, $cdoperador, $nrorg) {
        $this->connection->beginTransaction();
        try {
            $this->contratoService->excluirNegociacaoDeValoresContrato($nrseqcontrato, $nrorg);
            
            $itensContrato = $this->contratoService->getItensContratos($nrseqcontrato, $nrorg);
            foreach ($itensContrato as $item) {
                $this->inserirItemNegociacaoValoresContrato($item, $nrorg, $cdoperador, $nrtipoorcamento);
            }
            
            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }
    
    public function atualizaMunicipioNegociacaoValores($nrseqcontrato, $idmuniprestserv, $cdoperador, $nrorg) {
        $this->connection->beginTransaction();
        try {
            
            $params = array(
                "NRSEQCONTRATO" => $nrseqcontrato,
                "NRORG"         => $nrorg
            );
            $negociacoes = $this->itensFactory->getNegociacoesContrato($params);
            
            foreach ($negociacoes as $nego) {
            
                if($idmuniprestserv == 'U'){ //Unidade
                
                    $dadosEmpresa = $this->contratoFactory->getDadosEmpresa(array(
                        "CDFILIAL" => $nego["CDFILIALFATURA"],
                        "NRORG"    => $nrorg
                    ));
                    
                    if(count($dadosEmpresa) > 0 ){
                        $dadosEmpresa = $dadosEmpresa[0];
                        $cdpaisfat      = $dadosEmpresa["CDPAIS"];
                        $sgestadofat    = $dadosEmpresa["SGESTADO"];
                        $cdmunicipiofat = $dadosEmpresa["CDMUNICIPIO"];
                    }
                
                }else{ //Tomador - Preencher com o endereçõ do ENDERECOPARC 
                
                    $enderecofaturamento = $this->contratoService->getEnderecoFaturamento($nego["NRESTRUTURAM"], $nego["NRPARCFATURA"], $nrorg);
                 
                    $cdpaisfat      = $enderecofaturamento["CDPAIS"];
                    $sgestadofat    = $enderecofaturamento["SGESTADO"];
                    $cdmunicipiofat = $enderecofaturamento["CDMUNICIPIO"];
                }
                
                $params = array(
                    "CDPAISFAT"            => $cdpaisfat,
                    "SGESTADOFAT"          => $sgestadofat,
                    "CDMUNICIPIOFAT"       => $cdmunicipiofat,
                    "OPERADOR"             => $cdoperador,
                    "NRORG"                => $nrorg,
                    "NRNEGOCIACAOCONTRATO" => $nego["NRNEGOCIACAOCONTRATO"]
                );
                
                $this->itensFactory->atualizaMunicipioNegociacaoValores($params);
            }
            
            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }
}
?>